module StreetCreds
  class CredFile
    def self.encrypt_existing_file(filepath:)
      filepath = full_path(filepath)
      hash     = YAML.load(File.open(filepath))
      new(filepath: filepath, hash: hash)
    end

    def self.decrypt_existing_file(filepath:)
      filepath           = full_path(filepath)
      decrypted_contents = new(filepath: filepath).decrypt_file
      File.open(filepath, 'w+') { |f| f.write(decrypted_contents) }
    end

    def initialize(filepath:, hash: nil, convert_on_valid: false, password: nil)
      @filepath = self.class.full_path(filepath)
      @password = password
      load_file(hash: hash, convert_on_valid: convert_on_valid)
    end

    def load_file(convert_on_valid:, hash: nil)
      if hash.nil?
        begin
          file_contents = File.read(@filepath)
          return @hash = {} if file_contents.empty?
          @hash = YAMLHelper.load_if_valid(file_contents) if convert_on_valid
          @hash = YAMLHelper.load_if_valid(decrypt_file) unless @hash
        rescue Errno::ENOENT
          @hash = {}
        end
      end
      @hash = self.class.symbolize_keys(@hash)
    end

    def decrypt_file(contents: nil, password: nil)
      contents = File.read(@filepath) unless contents
      password = ask_password if password.nil?
      contents.decrypt(:symmetric, :password => password)
    end

    def save
      FileUtils.mkdir_p(File.dirname(@filepath))
      yaml = self.class.symbolize_keys(@hash).to_yaml
      if yaml.empty?
        File.delete(@filepath)
      else
        encrypted_yaml = encrypt_file(contents: yaml)
        File.open(@filepath, 'w+') { |f| f.write(encrypted_yaml) }
      end
    end

    def encrypt_file(contents:, password: nil)
      password = ask_password if password.nil?
      contents.encrypt(:symmetric, :password => password)
    end

    def ask_password
      return @password if @password
      print "Password?\n> "
      @password = $stdin.noecho(&:gets).chomp
      puts ''
      @password
    end

    def inspect
      @hash.inspect
    end

    # Act like a hash
    def method_missing(meth, *args)
      @hash.send(meth, *args)
    end

    def self.full_path(filepath)
      Pathname.new(filepath).expand_path.to_s
    end

    # Adapted from https://stackoverflow.com/a/8379653/5637619
    def self.symbolize_keys(hash)
      Hash[hash.map do |k, v|
        [k.to_sym, (v.is_a?(Hash) ? symbolize_keys(v) : v)]
      end]
    end
  end
end
